/**
  ******************************************************************************
  * @file    Project/main.c
  * @author  "Vladimir N. Shilov" <shilow@ukr.net>
  * @version v.0.2
  * @date    08-July-2019
  * @brief   Main program body
   ******************************************************************************
  * @attention
  *
  * "THE BEER-WARE LICENSE" (Revision 42):
  * <shilow@ukr.net> wrote this file.  As long as you retain this notice you
  * can do whatever you want with this stuff. If we meet some day, and you think
  * this stuff is worth it, you can buy me a beer in return.   Shilov V.N.
  *
  ******************************************************************************
  */


/* Includes ------------------------------------------------------------------*/
#include "stm8s.h"
#include "rtos.h"
#include "max7219.h"
#include "i2c.h"
#include "ina219.h"

/* Private defines -----------------------------------------------------------*/
const static max7219_reg_t digitPosition[8] = {
  RegDigit0, RegDigit1, RegDigit2, RegDigit3,
  RegDigit4, RegDigit5, RegDigit6, RegDigit7
};
const static max7219_sym_t digitValue[16] = {
  Sym_0, Sym_1, Sym_2, Sym_3, Sym_4, Sym_5, Sym_6, Sym_7,
  Sym_8, Sym_9, Sym_A, Sym_b, Sym_c, Sym_d, Sym_E, Sym_F
};

/* Private function prototypes -----------------------------------------------*/
static void boardInit(void);
static void showVoltage(void);

/* Private functions ---------------------------------------------------------*/

void main(void)
{
  boardInit();

  RTOS_Init();
  enableInterrupts();

  MAX7219_Config();

  MAX7219_WriteData(digitPosition[0], Sym_BLANK);
  MAX7219_WriteData(digitPosition[1], Sym_BLANK);
  MAX7219_WriteData(digitPosition[2], Sym_BLANK);
  MAX7219_WriteData(digitPosition[3], Sym_BLANK);
  MAX7219_WriteData(digitPosition[4], Sym_BLANK);
  MAX7219_WriteData(digitPosition[5], Sym_BLANK);
  MAX7219_WriteData(digitPosition[6], Sym_BLANK);
  MAX7219_WriteData(digitPosition[7], Sym_BLANK);

  if (CLK_GetSYSCLKSource() == CLK_SOURCE_HSE) {
    MAX7219_WriteData(digitPosition[3], Sym_E);
  } else {
    MAX7219_WriteData(digitPosition[3], Sym_i);
  }

  i2c_master_init();
  INA219_Config();

  RTOS_SetTask(showVoltage, 70, 250);

  /* Infinite loop */
  while (1)
  {
    wfi();
  }

}

/* Private functions ---------------------------------------------------------*/
static void boardInit(void) {
  /*   ,    16       FLASH:
    - Before using the HSE clock make sure that the "Flash_Wait_States" is set to 1.
    - To do so :
     - with STVD (menu: Debug Instrument -> MCU configuration -> Options)
     - with EWSTM8 (menu: ST-LINK -> Option bytes ->Flash_Wait_States: 1)
  */

  ErrorStatus status = ERROR;

  /* Initialization of the clock to 16MHz */
  CLK->CKDIVR = 0x00;

  /* Disable clock of unused peripherial */
  CLK->PCKENR1 = (uint8_t)(~CLK_PCKENR1_UART1);
  CLK->PCKENR2 = (uint8_t)(~(CLK_PCKENR2_CAN | CLK_PCKENR2_ADC | CLK_PCKENR2_AWU));

  /* Configure the system clock to use HSE clock source and to run at Crystal Mhz */
  status = CLK_ClockSwitchConfig(CLK_SWITCHMODE_AUTO, CLK_SOURCE_HSE, DISABLE, CLK_CURRENTCLOCKSTATE_DISABLE);
  if (status == ERROR) {
    /* Configure the system clock to use HSI clock source and to run at 16Mhz */
    // set FLAG for
  }

  /* Init GPIO */
  /* Set the MOSI,MISO and SCK at high level */
  SPI_PORT->ODR |= SPI_MOSI;
  SPI_PORT->DDR |= SPI_PINS;
  SPI_PORT->CR1 |= SPI_PINS;
  SPI_PORT->CR2 |= SPI_PINS;

  /* Configure LOAD pin to Push-Pull, High, Fast*/
  SPI_LOAD_PORT->ODR |= SPI_LOAD;
  SPI_LOAD_PORT->DDR |= SPI_LOAD;
  SPI_LOAD_PORT->CR1 |= SPI_LOAD;
  SPI_LOAD_PORT->CR2 |= SPI_LOAD;

  /* Init SPI */
  /* SPI_MODE_MASTER, SPI_FIRSTBIT_MSB, SPI_BAUDRATEPRESCALER_2, SPI_CLOCKPOLARITY_LOW, SPI_CLOCKPHASE_1EDGE */
  SPI->CR1 = 0x04;
  /* SPI_DATADIRECTION_1LINE_TX, SPI_NSS_SOFT */
  SPI->CR2 = 0xC0 | 0x02 | 0x01;
  /* SPI Enable */
  SPI->CR1 |= SPI_CR1_SPE;

}

/** */
static void showVoltage(void) {
  uint8_t c = 0;

  uint16_t volt = readBusVoltage();

  if (volt < 10000) {
    MAX7219_WriteData(digitPosition[2], digitValue[0]);
  } else {
    while (volt >= 10000) {
      volt -= 10000;
      c ++;
    }
    MAX7219_WriteData(digitPosition[3], digitValue[c]);
  }

  if (volt < 1000) {
    MAX7219_WriteData(digitPosition[3], digitValue[0]);
  } else {
    c = 0;
    while (volt >= 1000) {
      volt -= 1000;
      c ++;
    }
    MAX7219_WriteData(digitPosition[4], digitValue[c] | Sym_Dot);
  }

  if (volt < 100) {
    MAX7219_WriteData(digitPosition[5], digitValue[0]);
  } else {
    c = 0;
    while (volt >= 100) {
      volt -= 100;
      c ++;
    }
    MAX7219_WriteData(digitPosition[5], digitValue[c]);
  }

  if (volt < 10) {
    MAX7219_WriteData(digitPosition[6], digitValue[0]);
  } else {
    c = 0;
    while (volt >= 10) {
      volt -= 10;
      c ++;
    }
    MAX7219_WriteData(digitPosition[6], digitValue[c]);
  }

  MAX7219_WriteData(digitPosition[7], digitValue[volt]);
}

#ifdef USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
  */
void assert_failed(u8* file, u32 line)
{
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while (1)
  {
  }
}
#endif


/************************ (C) COPYRIGHT Shilov V.N. *****END OF FILE****/
